/******************************************************************************
 *                                                                            *
 *                                Copyright by                                *
 *                                                                            *
 *                              Azoteq (Pty) Ltd                              *
 *                          Republic of South Africa                          *
 *                                                                            *
 *                           Tel: +27(0)21 863 0033                           *
 *                           E-mail: info@azoteq.com                          *
 *                                                                            *
 *============================================================================*
 * @file        iqs7211a-example-code.ino                                     *
 * @brief       IQS7211A EV-KIT Variants Example code                         *
 *              This example demonstrates how to write the desired settings   *
 *              to the IQS7211A in order to use the IQS7211A EV-Kit variants. *
 *                                                                            *
 *              EV-KIT selection in IQS7211A.h:                               *
 *              0: Rectangle Trackpad EV-Kit (AZP1189A3).                     *
 *              1: Flower Trackpad EV-Kit (AZP1191A3).                        *
 *                                                                            *
 *              All data is displayed over serial communication with the      *
 *              following outputs:                                            *
 *                - Finger 1 X and Y Coordinates                              *
 *                - Trackpad Gestures                                         *
 *                - Power Mode Switching (if enabled)                         *
 *                - Force Communication when 'f\n' is sent over Serial        *
 *                - Software Reset when 'r\n' is sent over Serial             *
 *                                                                            *
 * @author      Azoteq PTY Ltd                                                *
 * @version     v1.2                                                          *
 * @date        2023-07-14                                                    *
 ******************************************************************************/
#include <Arduino.h>
#include "src/IQS7211A/IQS7211A.h"

/*** Defines ***/
#define DEMO_IQS7211A_ADDR        0x56
#define DEMO_IQS7211A_POWER_PIN   4
#define DEMO_IQS7211A_RDY_PIN     7

/*** Instances ***/
IQS7211A iqs7211a;

/*** Global Variables ***/
bool show_data = false;
iqs7211a_power_modes running_power_mode = IQS7211A_IDLE;
uint16_t running_x_output = 65535;
uint16_t running_y_output = 65535;
iqs7211a_gestures_e running_gestures = IQS7211A_GESTURE_NONE;

void setup()
{
  /* Start Serial Communication */
  Serial.begin(115200);
  while(!Serial);
  Serial.println("Start Serial communication");

  /* Power On IQS7211A */
  pinMode(DEMO_IQS7211A_POWER_PIN, OUTPUT);
  delay(200);
  digitalWrite(DEMO_IQS7211A_POWER_PIN, LOW);
  delay(200);
  digitalWrite(DEMO_IQS7211A_POWER_PIN, HIGH);

  #if IQS7211A_EV_KIT == 0
    Serial.println("IQS7211A Rectangle Touchpad EV-Kit Selected");
  #elif IQS7211A_EV_KIT == 1
    Serial.println("IQS7211A Flower Touchpad EV-Kit Selected");
  #endif

  /* Initialize the IQS7211A with input parameters device address and RDY pin */
  iqs7211a.begin(DEMO_IQS7211A_ADDR, DEMO_IQS7211A_RDY_PIN);
  Serial.println("IQS7211A Ready");
  delay(1);
}

void loop()
{
  /* Read new data from IQS7211A if available  (RDY Line Low) */
  iqs7211a.run();

  /* Function to initialize a force communication window. */
  force_comms_and_reset();

  /* If data was updated, display data read from IQS7211A */
  if (iqs7211a.new_data_available)
  {
    /* Print the following if the new data did not come from a force comms command */
    if (!show_data)
    {
      if (printData())
      {
        printGesture();
        printCoordinates();
        printPowerMode();
      }
    }
    /* Display the heading when a finger is lifted from the trackpad */
    printHeading();
    /* Set this flag to false to indicate that the new data was already displayed/used */
    iqs7211a.new_data_available = false;
    /* Check if a force comms command was sent and if we should display the data read */
    show_iqs7211a_data();
  }
}

/* Check if one of the gesture flags is set and display which one */
void printGesture(void)
{
  iqs7211a_gestures_e tempGestures = iqs7211a.get_gesture_event();

  switch (tempGestures)
  {
    case IQS7211A_GESTURE_SINGLE_TAP:
      Serial.print("Single Tap\t\t");
    break;

    case IQS7211A_GESTURE_PRESS_HOLD:
      Serial.print("Press and Hold\t\t");
    break;

    case IQS7211A_GESTURE_SWIPE_X_POSITIVE:
      Serial.print("Swipe X +\t\t");
    break;

    case IQS7211A_GESTURE_SWIPE_X_NEGATIVE:
      Serial.print("Swipe X -\t\t");
    break;

    case IQS7211A_GESTURE_SWIPE_Y_POSITIVE:
      Serial.print("Swipe Y +\t\t");
    break;

    case IQS7211A_GESTURE_SWIPE_Y_NEGATIVE:
      Serial.print("Swipe Y -\t\t");
    break;

    default:
      Serial.print("-\t\t\t");
    break;
  }

  /* Update the running values for the next print heading call */
  running_gestures = tempGestures;
}

/* Check Power mode and print out the current power mode */
void printPowerMode(void)
{
  iqs7211a_power_modes buffer = iqs7211a.getPowerMode(); // read current power mode

  switch (buffer)
  {
    case IQS7211A_ACTIVE:
      Serial.println("Active");
      break;

    case IQS7211A_IDLE_TOUCH:
      Serial.println("Idle-Touch");
      break;

    case IQS7211A_IDLE:
      Serial.println("Idle");
      break;

    case IQS7211A_LP1:
      Serial.println("Low Power 1");
      break;

    case IQS7211A_LP2:
      Serial.println("Low Power 2");
      break;

    default:
      Serial.println("Unknown");
      break;
    }

    /* Update the running state */
    running_power_mode = buffer;
}

/** Function to print heading on the release of a previous event
 *  See if there are no fingers on the trackpad
 */
void printHeading(void)
{
  /* Check if it is necessary to display the heading */
  if (iqs7211a.getNumFingers() == 0)
  {
    Serial.println("\nGesture:\t\tFinger 1 X:\tFinger 1 Y:\tPower Mode:");
  }
}

/* Function to determine if it is necessary to print all the relevant data 
in the serial terminal */
bool printData(void)
{
/* See if it is necessary to display the button state, power mode or gestures */
  if(
      (
        (iqs7211a.getPowerMode()              != running_power_mode)
      ||(iqs7211a.get_gesture_event()         != running_gestures)
      ||(iqs7211a.getAbsXCoordinate(FINGER_1) != running_x_output)
      ||(iqs7211a.getAbsYCoordinate(FINGER_1) != running_y_output)
      )
    )
  {
    return true; // Let the main loop know to display the latest values
  }
  else
  {
    return false; // Let the main loop know to not display the latest values
  }

  return false;
}

/* Function to return X and Y coordinates of finger 1 */
void printCoordinates(void)
{
  uint16_t ui16TempX = iqs7211a.getAbsXCoordinate(FINGER_1);
  uint16_t ui16TempY = iqs7211a.getAbsYCoordinate(FINGER_1);

  Serial.print(ui16TempX); // Print X coordinates
  Serial.print("\t\t");    // Spacing to match headings in Print
  Serial.print(ui16TempY); // Print Y coordinates
  Serial.print("\t\t");    // Spacing to match headings in Print

  running_x_output = ui16TempX;
  running_y_output = ui16TempY;
}

/* Force the IQS7211A to open a RDY window and read the current state of the device */
void force_comms_and_reset(void)
{
  char message = read_message();

  /* If an 'f' was received over serial, open a forced communication window and
  print the new data received */
  if(message == 'f')
  {
    iqs7211a.force_I2C_communication(); // prompt the IQS7211A
    show_data = true;
  }

  /* If an 'r' was received over serial, request a software(SW) reset */
  if(message == 'r')
  {
    Serial.println("Software Reset Requested!");
    iqs7211a.force_I2C_communication(); // Request a RDY window
    iqs7211a.iqs7211a_state.state = IQS7211A_STATE_SW_RESET;
    running_gestures = IQS7211A_GESTURE_NONE;
    running_x_output = 65535;
    running_y_output = 65535;
    running_power_mode = IQS7211A_POWER_UNKNOWN;
  }
}

/* Read message sent over serial communication */
char read_message(void)
{
  while (Serial.available())
  {
    if (Serial.available() > 0)
    {
      return Serial.read();
    }
  }

  return '\n';
}

/* Shows the current IQS7211A data when a force comms command was sent */
void show_iqs7211a_data()
{
  if (show_data)
  {
    Serial.println("\n**************************************************************************");
    Serial.println("********************* IQS7211A DATA - FORCED COMMS ***********************");
    Serial.println("**************************************************************************");
    Serial.println("Gesture:\t\tFinger 1 X:\tFinger 1 Y:\tPower Mode:");
    printGesture();
    printCoordinates();
    printPowerMode();
    Serial.println("**************************************************************************");
    show_data = false;
  }
}